package com.jeecms.common.upload;

import com.qcloud.cos.COSClient;
import com.qcloud.cos.ClientConfig;
import com.qcloud.cos.auth.BasicCOSCredentials;
import com.qcloud.cos.auth.COSCredentials;
import com.qcloud.cos.http.HttpProtocol;
import com.qcloud.cos.model.COSObjectSummary;
import com.qcloud.cos.model.CopyObjectRequest;
import com.qcloud.cos.model.ListObjectsRequest;
import com.qcloud.cos.model.ObjectListing;
import com.qcloud.cos.model.ObjectMetadata;
import com.qcloud.cos.model.PutObjectRequest;
import com.qcloud.cos.model.PutObjectResult;
import com.qcloud.cos.model.StorageClass;
import com.qcloud.cos.region.Region;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * 腾讯COS上传工具
 * 
 * @author: tom
 * @date: 2018年4月14日 下午4:00:16
 * @Copyright: 杭州藏茗山科技有限公司 All rights reserved. Notice
 *             仅限于授权后使用，禁止非授权传阅以及私自用于商业目的。
 */
public class QcoscloudClient {
	private static final Logger log = LoggerFactory.getLogger(QcoscloudClient.class);

	/**
	 * 本地文件上传腾讯cos
	 * @Title: uploadFileByLocal
	 * @param bucketArea bucketArea
	 * @param bucketName bucketName
	 * @param appId appId
	 * @param secretId  secretId
	 * @param appKey appKey
	 * @param cosPath  远程文件夹的名称
	 * @param localPath 文件的本地路径
	 * @return: String
	 */
	public static String uploadFileByLocal(String bucketArea, String bucketName, String appId, String secretId,
			String appKey, String cosPath, String localPath) {
		String result = "";
		COSClient cosClient = createCOSClient(bucketArea, appId, secretId, appKey);
		if (cosClient != null) {
			try {
				File localFile = new File(localPath);
				PutObjectRequest putObjectRequest = new PutObjectRequest(bucketName, cosPath, localFile);
				putObjectRequest.setStorageClass(StorageClass.Standard_IA);
				cosClient.putObject(putObjectRequest);
				// 确认本进程不再使用 cosClient 实例之后，关闭
				cosClient.shutdown();
			}catch (Exception e){
				log.error("腾讯云本地文件上传失败。",e);
				result = "error";
			}
		}
		return result;
	}

	/**
	 * 以字节数组的形式上传文件
	 * @Title: uploadFileByByte
	 * @param bucketArea bucketArea
	 * @param bucketName bucketName
	 * @param appId appId
	 * @param secretId secretId
	 * @param appKey appKey
	 * @param cosPath  目标路径
	 * @param content 字节数组
	 * @return: String
	 */
	public static String uploadFileByByte(String bucketArea, String bucketName, String appId, String secretId,
			String appKey, String cosPath, byte[] content) {
		String result = "";
		COSClient cosClient = createCOSClient(bucketArea, appId, secretId, appKey);
		if (cosClient != null) {
			// 字节数组转成流上传
			InputStream inputStream = new ByteArrayInputStream(content);
			result = uploadFileByInputStream(bucketArea,bucketName,appId,secretId,appKey,cosPath,inputStream);
		}
		return result;
	}

	/**
	 * 上传文件 以输入流的形式
	 * @Title: uploadFileByInputStream
	 * @param bucketArea bucketArea
	 * @param bucketName bucketName
	 * @param appId appId
	 * @param secretId secretId
	 * @param appKey appKey
	 * @param cosPath 目标路径
	 * @param inputStream 输入流
	 * @return: String
	 */
	public static String uploadFileByInputStream(String bucketArea, String bucketName,
			String appId, String secretId,
			String appKey, String cosPath, InputStream inputStream) {
		String result = "";
		COSClient cosClient = createCOSClient(bucketArea, appId, secretId, appKey);
		if (cosClient != null) {
			ObjectMetadata objectMetadata = new ObjectMetadata();
			// 上传的流如果能够获取准确的流长度，则推荐一定填写 content-length
			// 如果确实没办法获取到，则下面这行可以省略，但同时高级接口也没办法使用分块上传了
			// objectMetadata.setContentLength(0);
			PutObjectRequest putObjectRequest = new PutObjectRequest(bucketName, cosPath, inputStream, objectMetadata);
			// 设置存储类型（如有需要，不需要请忽略此行代码）, 默认是标准(Standard), 低频(standard_ia)
			putObjectRequest.setStorageClass(StorageClass.Standard_IA);
			PutObjectResult putObjectResult = null;
			try {
				// 返回腾讯云的目标路径
				result = cosPath;
				cosClient.putObject(putObjectRequest);
				// 确认本进程不再使用 cosClient 实例之后，关闭
				cosClient.shutdown();
			}catch (Exception e){
				log.error("腾讯云文件上传失败。",e);
				result = "";
			}
		}
		return result;
	}

	/**
	 * 移动文件
	 * @Title: moveFile
	 * @param bucketArea bucketArea
	 * @param bucketName bucketName
	 * @param appId appId
	 * @param secretId  secretId
	 * @param appKey appKey
	 * @param cosFilePath  原路径
	 * @param dstCosFilePath 目标路径
	 * @return: String
	 */
	public static String moveFile(String bucketArea, String bucketName, String appId,
			String secretId, String appKey,
			String cosFilePath, String dstCosFilePath) {
		String result = "";
		COSClient cosClient = createCOSClient(bucketArea, appId, secretId, appKey);
		if (cosClient != null) {
			try {
				Region srcBucketRegion = new Region(bucketArea);
				//  先复制源文件 到目标路径, 然后删除源文件
				CopyObjectRequest copyObjectRequest = new CopyObjectRequest(srcBucketRegion, bucketName,
						cosFilePath, bucketName, dstCosFilePath);
				// 复制文件
				cosClient.copyObject(copyObjectRequest);
				// 删除源文件
				cosClient.deleteObject(bucketName, cosFilePath);
				// 确认本进程不再使用 cosClient 实例之后，关闭
				cosClient.shutdown();
			}catch (Exception e){
				log.error("腾讯云 复制、删除文件出错。",e);
				result = "error";
			}
		}
		return result;
	}

	/**
	 * 删除文件
	 * @Title: deleteFile
	 * @param bucketArea bucketArea
	 * @param bucketName bucketName
	 * @param appId appId
	 * @param secretId secretId
	 * @param appKey appKey
	 * @param cosFilePath
	 *            目标文件路径
	 * @return: String
	 */
	public static String deleteFile(String bucketArea, String bucketName, 
			String appId, String secretId, String appKey,String cosFilePath) {
		String result = "";
		COSClient cosClient = createCOSClient(bucketArea, appId, secretId, appKey);
		if (cosClient != null) {
			try {
				cosClient.deleteObject(bucketName, cosFilePath);
				// 确认本进程不再使用 cosClient 实例之后，关闭
				cosClient.shutdown();
			}catch (Exception e){
				log.error("腾讯云删除文件异常。",e);
				result = "error";
			}
		}
		return result;
	}

	/**
	 * 查询指定文件夹下的文件
	 * 只查询子文件夹 未递归查询子文件夹
	 * 递归查询未实现
	 * @Title: listFolder
	 * @param bucketArea bucketArea
	 * @param bucketName bucketName
	 * @param appId appId
	 * @param secretId secretId
	 * @param appKey appKey
	 * @param cosPath  目标文件夹
	 * @return: List<String> 文件名称列表
	 */
	private static List<String> listFolder(String bucketArea, String bucketName,
			String appId, String secretId, String appKey,
			String cosPath) {
		List<String> pathFileStrList = new ArrayList<>();
		COSClient cosClient = createCOSClient(bucketArea, appId, secretId, appKey);
		if (cosClient != null) {
			try {
				ListObjectsRequest listObjectsRequest = new ListObjectsRequest();
				// 设置 bucket 名称
				listObjectsRequest.setBucketName(bucketName);
				// prefix 表示列出的对象名以 prefix 为前缀 这里填要列出的目录的相对 bucket 的路径 如: /dir/
				listObjectsRequest.setPrefix(cosPath);
				// delimiter 表示目录的截断符, 例如：设置为 / 则表示对象名遇到 / 就当做一级目录）
				listObjectsRequest.setDelimiter("/");
				// 设置最大遍历出多少个对象, 一次 listObject 最大支持1000 默认值： 1000
				listObjectsRequest.setMaxKeys(1000);
				// 保存每次列出的结果
				ObjectListing objectListing = cosClient.listObjects(listObjectsRequest);
				// 获取当前文件夹下面的文件夹
				// List<String> commonPrefixes = objectListing.getCommonPrefixes();
				// if(!CollectionUtils.isEmpty(commonPrefixes)){
				// 	pathFileStrList.addAll(commonPrefixes);
				// }
				// 获取当前文件夹下的文件
				List<COSObjectSummary> objectSummaries = objectListing.getObjectSummaries();
				if(!CollectionUtils.isEmpty(objectSummaries)){
					for(COSObjectSummary cosObjectSummary : objectSummaries){
						// 对象的key 也就是path文件名 如: u/cms/www/202210/19110517kqpz.jpg
						pathFileStrList.add(cosObjectSummary.getKey());
					}
				}
			}catch (Exception e){
				log.error("腾讯云获取置顶文件夹下内容异常。",e);
			}
		}
		return pathFileStrList;
	}

	/**
	 * 删除文件夹
	 * @Title: delFolder delFolder
	 * @param bucketArea bucketArea
	 * @param bucketName bucketName
	 * @param appId appId
	 * @param secretId secretId
	 * @param appKey appKey
	 * @param cosPath
	 *            删除目标文件夹路径
	 * @return: String
	 */
	public static String delFolder(String bucketArea, String bucketName, 
			String appId, String secretId, String appKey,String cosPath) {
		COSClient cosClient = createCOSClient(bucketArea, appId, secretId, appKey);
		if (cosClient != null) {
			// DelFolderRequest delFolderRequest = new DelFolderRequest(bucketName, cosPath);
			// String delFolderRet = cosClient.delFolder(delFolderRequest);
			// return delFolderRet;
			// todo  待完成
			return "";
		} else {
			return "";
		}
	}

	/**
	 * 获取文件夹信息
	 * @Title: statFolder
	 * @param bucketArea bucketArea
	 * @param bucketName bucketName
	 * @param appId appId
	 * @param secretId secretId
	 * @param appKey appKey
	 * @param cosPath cos路径
	 * @return: String
	 */
	public static String statFolder(String bucketArea, String bucketName, 
			String appId, String secretId, String appKey,String cosPath) {
		COSClient cosClient = createCOSClient(bucketArea, appId, secretId, appKey);
		if (cosClient != null) {
			// StatFolderRequest statFolderRequest = new StatFolderRequest(bucketName, cosPath);
			// String statFolderRet = cosClient.statFolder(statFolderRequest);
			// return statFolderRet;
			// todo  待完成
			return "";
		} else {
			return "";
		}
	}

	/**
	 * 获取腾讯COS Client
	 * @Title: getCOSClient
	 * @param bucketArea bucketArea
	 * @param appId  appId
	 * @param secretId secretId
	 * @param appKey appKey
	 * @return: COSClient
	 */
	private static COSClient createCOSClient(String bucketArea, String appId, String secretId, String appKey) {
		if (StringUtils.isNotBlank(appId) && StringUtils.isNumeric(appId)) {
			COSCredentials cred = new BasicCOSCredentials(secretId, appKey);
			ClientConfig clientConfig = new ClientConfig();
			// 设置 bucket 的地域
			clientConfig.setRegion(new Region(bucketArea));
			// 5.6.54 及更高版本，默认使用了 https
			clientConfig.setHttpProtocol(HttpProtocol.http);
			// 生成 cos 客户端。
			return new COSClient(cred, clientConfig);
		} else {
			log.error("createCOSClient error, 参数异常 appId->{} ,bucketArea->{},secretId->{},appKey->{}",appId,bucketArea,secretId,appKey);
			return null;
		}
	}

	/**
	 * 解析各个文件夹
	 * @param bucketArea
	 * @param bucketName
	 * @param appId
	 * @param secretId
	 * @param appKey
	 * @param path 上传目录（加 / 号）类似 /xx/
	 * @return
	 */
	public static List<String> parseList(String bucketArea, String bucketName,
										 String appId, String secretId, String appKey, String path) {
		/**云存储需要/开头*/
		if(!path.startsWith("/")){
			path = "/"+ path;
		}
		//返回path下的文件夹以及文件(未递归)
		return listFolder(bucketArea, bucketName, appId, secretId, appKey, path);
	}
}